/*
 * Copyright 1999-2018 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.csp.sentinel.dashboard.rule.nacos;

import com.alibaba.csp.sentinel.dashboard.common.NacosConstants;
import com.alibaba.csp.sentinel.dashboard.datasource.entity.rule.DegradeRuleEntity;
import com.alibaba.csp.sentinel.dashboard.datasource.entity.rule.FlowRuleEntity;
import com.alibaba.csp.sentinel.dashboard.rule.DynamicRulePublisher;
import com.alibaba.csp.sentinel.datasource.Converter;
import com.alibaba.csp.sentinel.util.AssertUtil;
import com.alibaba.nacos.api.config.ConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Eric Zhao
 */
@Service
public class DegradeRuleNacosPublisher implements DynamicRulePublisher<List<DegradeRuleEntity>> {

    @Autowired
    private static Logger logger = LoggerFactory.getLogger(DegradeRuleNacosPublisher.class);
    @Autowired
    @Qualifier("nacosDegradeConfiguration")
    private ConfigService nacosDegradeConfiguration;
    @Autowired
    private NacosProperties nacosProperties;
    @Autowired
    private Converter<List<DegradeRuleEntity>,String> converter;

    @Override
    public void publish(String appName, List<DegradeRuleEntity> rules) throws Exception {

        logger.info("DegradeRuleNacosPublisher publish [appName] " + appName);
        AssertUtil.notEmpty(appName, "appName不能为空");
        if(rules == null) {
            return;
        }
        String dataId=new StringBuilder(appName).append(NacosConstants.DATA_ID_DEGRADE_POSTFIX).toString();
        nacosDegradeConfiguration.publishConfig(dataId,nacosProperties.getGroupId(),converter.convert(rules));
    }
}
